@@ -31,6 +31,7 @@ class Table(object):
         table_name,
         primary_id=None,
         primary_type=None,
+        primary_increment=None,
         auto_create=False,
     ):
         """Initialise the table from database schema."""
@@ -43,6 +44,9 @@ class Table(object):
             primary_id if primary_id is not None else self.PRIMARY_DEFAULT
         )
         self._primary_type = primary_type if primary_type is not None else Types.integer
+        if primary_increment is None:
+            primary_increment = self._primary_type in (Types.integer, Types.bigint)
+        self._primary_increment = primary_increment
         self._auto_create = auto_create
 
     @property
@@ -264,28 +268,15 @@ class Table(object):
     def upsert_many(self, rows, keys, chunk_size=1000, ensure=None, types=None):
         """
         Sorts multiple input rows into upserts and inserts. Inserts are passed
-        to insert_many and upserts are updated.
+        to insert and upserts are updated.
 
         See :py:meth:`upsert() <dataset.Table.upsert>` and
         :py:meth:`insert_many() <dataset.Table.insert_many>`.
         """
-        keys = ensure_list(keys)
-
-        to_insert = []
-        to_update = []
+        # Removing a bulk implementation in 5e09aba401. Doing this one by one
+        # is incredibly slow, but doesn't run into issues with column creation.
         for row in rows:
-            if self.find_one(**{key: row.get(key) for key in keys}):
-                # Row exists - update it.
-                to_update.append(row)
-            else:
-                # Row doesn't exist - insert it.
-                to_insert.append(row)
-
-        # Insert non-existing rows.
-        self.insert_many(to_insert, chunk_size, ensure, types)
-
-        # Update existing rows.
-        self.update_many(to_update, keys, chunk_size, ensure, types)
+            self.upsert(row, keys, ensure=ensure, types=types)
 
     def delete(self, *clauses, **filters):
         """Delete rows from the table.
@@ -343,14 +334,11 @@ class Table(object):
                 if self._primary_id is not False:
                     # This can go wrong on DBMS like MySQL and SQLite where
                     # tables cannot have no columns.
-                    primary_id = self._primary_id
-                    primary_type = self._primary_type
-                    increment = primary_type in [Types.integer, Types.bigint]
                     column = Column(
-                        primary_id,
-                        primary_type,
+                        self._primary_id,
+                        self._primary_type,
                         primary_key=True,
-                        autoincrement=increment,
+                        autoincrement=self._primary_increment,
                     )
                     self._table.append_column(column)
                 for column in columns:
@@ -414,7 +402,7 @@ class Table(object):
             return self.table.c[column] == value
         if op in ("!=", "<>", "not"):
             return self.table.c[column] != value
-        if op in ("in"):
+        if op in ("in",):
             return self.table.c[column].in_(value)
         if op in ("between", ".."):
             start, end = value
@@ -498,9 +486,12 @@ class Table(object):
         self.create_column(name, type_)
 
     def drop_column(self, name):
-        """Drop the column ``name``.
+        """
+        Drop the column ``name``.
         ::
+
             table.drop_column('created_at')
+
         """
         if self.db.engine.dialect.name == "sqlite":
             raise RuntimeError("SQLite does not support dropping columns.")
@@ -601,9 +592,12 @@ class Table(object):
             # return all rows sorted by multiple columns (descending by year)
             results = table.find(order_by=['country', '-year'])
 
-        To perform complex queries with advanced filters or to perform
-        aggregation, use :py:meth:`db.query() <dataset.Database.query>`
-        instead.
+        You can also submit filters based on criteria other than equality,
+        see :ref:`advanced_filters` for details.
+
+        To run more complex queries with JOINs, or to perform GROUP BY-style
+        aggregation, you can also use :py:meth:`db.query() <dataset.Database.query>`
+        to run raw SQL queries instead.
         """
         if not self.exists:
             return iter([])
